@@ -20,7 +20,8 @@ SAVE_CURSOR = '\0337'
 RESTORE_CURSOR = '\0338'
 SAVE_PRIVATE_MODE_VALUES = '\033[?s'
 RESTORE_PRIVATE_MODE_VALUES = '\033[?r'
-
+SAVE_COLORS = '\033[#P'
+RESTORE_COLORS = '\033[#Q'
 MODES = dict(
     LNM=(20, ''),
     IRM=(4, ''),
@@ -39,7 +40,6 @@ MODES = dict(
     MOUSE_URXVT_MODE=(1015, '?'),
     ALTERNATE_SCREEN=(1049, '?'),
     BRACKETED_PASTE=(2004, '?'),
-    EXTENDED_KEYBOARD=(2017, '?'),
 )
 
 F = TypeVar('F')
@@ -68,6 +68,11 @@ def clear_screen() -> str:
     return '\033[H\033[2J'
 
 
+@cmd
+def clear_to_end_of_screen() -> str:
+    return '\033[J'
+
+
 @cmd
 def clear_to_eol() -> str:
     return '\033[K'
@@ -108,6 +113,12 @@ def set_cursor_position(x: int, y: int) -> str:  # (0, 0) is top left
     return '\033[{};{}H'.format(y + 1, x + 1)
 
 
+@cmd
+def move_cursor_by(amt: int, direction: str) -> str:
+    suffix = {'up': 'A', 'down': 'B', 'right': 'C', 'left': 'D'}[direction]
+    return f'\033[{amt}{suffix}'
+
+
 @cmd
 def set_cursor_shape(shape: str = 'block', blink: bool = True) -> str:
     val = {'block': 1, 'underline': 3, 'bar': 5}.get(shape, 1)
@@ -253,28 +264,29 @@ def init_state(alternate_screen: bool = True) -> str:
     ans = (
         S7C1T + SAVE_CURSOR + SAVE_PRIVATE_MODE_VALUES + reset_mode('LNM') +
         reset_mode('IRM') + reset_mode('DECKM') + reset_mode('DECSCNM') +
-        set_mode('DECARM') + reset_mode('DECOM') + set_mode('DECAWM') +
+        set_mode('DECARM') + set_mode('DECAWM') +
         set_mode('DECTCEM') + reset_mode('MOUSE_BUTTON_TRACKING') +
         reset_mode('MOUSE_MOTION_TRACKING') + reset_mode('MOUSE_MOVE_TRACKING') +
         reset_mode('FOCUS_TRACKING') + reset_mode('MOUSE_UTF8_MODE') +
         reset_mode('MOUSE_SGR_MODE') + reset_mode('MOUSE_UTF8_MODE') +
-        set_mode('BRACKETED_PASTE') + set_mode('EXTENDED_KEYBOARD') +
-        '\033]30001\033\\' +
+        set_mode('BRACKETED_PASTE') + SAVE_COLORS +
         '\033[*x'  # reset DECSACE to default region select
     )
     if alternate_screen:
-        ans += set_mode('ALTERNATE_SCREEN')
+        ans += set_mode('ALTERNATE_SCREEN') + reset_mode('DECOM')
         ans += clear_screen()
+    ans += '\033[>31u'  # extended keyboard mode
     return ans
 
 
 def reset_state(normal_screen: bool = True) -> str:
     ans = ''
+    ans += '\033[<u'  # restore keyboard mode
     if normal_screen:
         ans += reset_mode('ALTERNATE_SCREEN')
     ans += RESTORE_PRIVATE_MODE_VALUES
     ans += RESTORE_CURSOR
-    ans += '\033]30101\033\\'
+    ans += RESTORE_COLORS
     return ans
 
 
@@ -340,24 +352,17 @@ def set_default_colors(
 
 @cmd
 def write_to_clipboard(data: Union[str, bytes], use_primary: bool = False) -> str:
-    if isinstance(data, str):
-        data = data.encode('utf-8')
     from base64 import standard_b64encode
     fmt = 'p' if use_primary else 'c'
-
-    def esc(chunk: str) -> str:
-        return '\x1b]52;{};{}\x07'.format(fmt, chunk)
-
-    ans = esc('!')  # clear clipboard buffer
-    for chunk in (data[i:i+512] for i in range(0, len(data), 512)):
-        s = standard_b64encode(chunk).decode('ascii')
-        ans += esc(s)
-    return ans
+    if isinstance(data, str):
+        data = data.encode('utf-8')
+    payload = standard_b64encode(data).decode('ascii')
+    return f'\x1b]52;{fmt};{payload}\a'
 
 
 @cmd
 def request_from_clipboard(use_primary: bool = False) -> str:
-    return '\x1b]52;{};?\x07'.format('p' if use_primary else 'c')
+    return '\x1b]52;{};?\a'.format('p' if use_primary else 'c')
 
 
 # Boilerplate to make operations available via Handler.cmd  {{{
