@@ -14,7 +14,6 @@ from typing import (
 import kitty.fast_data_types as fast_data_types
 
 from .constants import is_macos, shell_path, terminfo_dir
-from .options_stub import Options
 
 try:
     from typing import TypedDict
@@ -156,9 +155,12 @@ def default_env() -> Dict[str, str]:
 
 def set_default_env(val: Optional[Dict[str, str]] = None) -> None:
     env = process_env().copy()
+    has_lctype = False
     if val:
+        has_lctype = 'LC_CTYPE' in val
         env.update(val)
     setattr(default_env, 'env', env)
+    setattr(default_env, 'lc_ctype_set_by_user', has_lctype)
 
 
 def openpty() -> Tuple[int, int]:
@@ -184,7 +186,6 @@ class Child:
         self,
         argv: Sequence[str],
         cwd: str,
-        opts: Options,
         stdin: Optional[bytes] = None,
         env: Optional[Dict[str, str]] = None,
         cwd_from: Optional[int] = None,
@@ -201,7 +202,6 @@ class Child:
         else:
             cwd = os.path.expandvars(os.path.expanduser(cwd or os.getcwd()))
         self.cwd = os.path.abspath(cwd)
-        self.opts = opts
         self.stdin = stdin
         self.env = env or {}
 
@@ -210,8 +210,11 @@ class Child:
         env: Optional[Dict[str, str]] = getattr(self, '_final_env', None)
         if env is None:
             env = self._final_env = default_env().copy()
+            if is_macos and env.get('LC_CTYPE') == 'UTF-8' and not sys._xoptions.get(
+                    'lc_ctype_before_python') and not getattr(default_env, 'lc_ctype_set_by_user', False):
+                del env['LC_CTYPE']
             env.update(self.env)
-            env['TERM'] = self.opts.term
+            env['TERM'] = fast_data_types.get_options().term
             env['COLORTERM'] = 'truecolor'
             if self.cwd:
                 # needed in case cwd is a symlink, in which case shells
