@@ -12,7 +12,7 @@ from typing import (
 
 from .cli_stub import CLIOptions
 from .conf.utils import resolve_config
-from .constants import appname, defconf, is_macos, str_version
+from .constants import appname, defconf, is_macos, str_version, website_url
 from .options.types import Options as KittyOpts
 from .typing import BadLineType, TypedDict
 
@@ -57,42 +57,60 @@ def surround(x: str, start: int, end: int) -> str:
     return x
 
 
+role_map: Dict[str, Callable[[str], str]] = {}
+
+
+def role(func: Callable[[str], str]) -> Callable[[str], str]:
+    role_map[func.__name__] = func
+    return func
+
+
+@role
 def emph(x: str) -> str:
     return surround(x, 91, 39)
 
 
+@role
 def cyan(x: str) -> str:
     return surround(x, 96, 39)
 
 
+@role
 def green(x: str) -> str:
     return surround(x, 32, 39)
 
 
+@role
 def blue(x: str) -> str:
     return surround(x, 34, 39)
 
 
+@role
 def yellow(x: str) -> str:
     return surround(x, 93, 39)
 
 
+@role
 def italic(x: str) -> str:
     return surround(x, 3, 23)
 
 
+@role
 def bold(x: str) -> str:
     return surround(x, 1, 22)
 
 
+@role
 def title(x: str) -> str:
     return blue(bold(x))
 
 
+@role
 def opt(text: str) -> str:
     return text
 
 
+@role
 def option(x: str) -> str:
     idx = x.rfind('--')
     if idx < 0:
@@ -103,24 +121,32 @@ def option(x: str) -> str:
     return ' '.join(parts)
 
 
+@role
 def code(x: str) -> str:
     return x
 
 
+@role
 def kbd(x: str) -> str:
     return x
 
 
+@role
 def env(x: str) -> str:
     return italic(x)
 
 
+role_map['envvar'] = role_map['env']
+
+
+@role
 def file(x: str) -> str:
     return italic(x)
 
 
+@role
 def doc(x: str) -> str:
-    return f'https://sw.kovidgoyal.net/kitty/{x}.html'
+    return website_url(x)
 
 
 OptionSpecSeq = List[Union[str, OptionDict]]
@@ -197,11 +223,13 @@ def parse_option_spec(spec: Optional[str] = None) -> Tuple[OptionSpecSeq, Option
 
 
 def prettify(text: str) -> str:
-    role_map = globals()
+
+    def identity(x: str) -> str:
+        return x
 
     def sub(m: Match) -> str:
         role, text = m.group(1, 2)
-        return str(role_map[role](text))
+        return role_map.get(role, identity)(text)
 
     text = re.sub(r':([a-z]+):`([^`]+)`', sub, text)
     return text
@@ -265,7 +293,8 @@ Run the :italic:`{appname}` terminal emulator. You can also specify the :italic:
 to run inside :italic:`{appname}` as normal arguments following the :italic:`options`.
 For example: {appname} sh -c "echo hello, world. Press ENTER to quit; read"
 
-For comprehensive documentation for kitty, please see: https://sw.kovidgoyal.net/kitty/''').format(appname=appname)
+For comprehensive documentation for kitty, please see: {url}''').format(
+    appname=appname, url=website_url())
 
 
 class PrintHelpForSeq:
